@@ -19,14 +19,14 @@ from __future__ import division, print_function, absolute_import
 
 import os
 import re
-import sqlite3
 import binascii
 import threading
 from contextlib import contextmanager
 
-from hypothesis.internal.compat import FileNotFoundError, sha1, \
+from hypothesis.internal.compat import FileNotFoundError, sha1, hbytes, \
     b64decode, b64encode
 
+sqlite3 = None
 SQLITE_PATH = re.compile(r"\.\(db|sqlite|sqlite3\)$")
 
 
@@ -100,10 +100,10 @@ class InMemoryExampleDatabase(ExampleDatabase):
             yield v
 
     def save(self, key, value):
-        self.data.setdefault(key, set()).add(value)
+        self.data.setdefault(key, set()).add(hbytes(value))
 
     def delete(self, key, value):
-        self.data.get(key, set()).discard(value)
+        self.data.get(key, set()).discard(hbytes(value))
 
     def close(self):
         pass
@@ -115,6 +115,8 @@ class SQLiteExampleDatabase(ExampleDatabase):
         self.path = path
         self.db_created = False
         self.current_connection = threading.local()
+        global sqlite3
+        import sqlite3
 
     def connection(self):
         if not hasattr(self.current_connection, 'connection'):
@@ -237,7 +239,7 @@ class DirectoryBasedExampleDatabase(ExampleDatabase):
         for path in os.listdir(kp):
             try:
                 with open(os.path.join(kp, path), 'rb') as i:
-                    yield i.read()
+                    yield hbytes(i.read())
             except FileNotFoundError:
                 pass
 
