@@ -38,7 +38,6 @@ from hypothesis.internal.renaming import renamed_arguments
 from hypothesis.utils.conventions import infer, not_set
 from hypothesis.internal.reflection import proxies, required_args
 from hypothesis.internal.branchcheck import check_function
-from hypothesis.searchstrategy.reprwrapper import ReprWrapperStrategy
 
 __all__ = [
     'nothing',
@@ -111,14 +110,24 @@ def cacheable(fn):
     return cached_strategy
 
 
-def defines_strategy(strategy_definition):
-    from hypothesis.searchstrategy.lazy import LazyStrategy
-    _strategies.add(strategy_definition.__name__)
+def base_defines_strategy(force_reusable):
+    def decorator(strategy_definition):
+        from hypothesis.searchstrategy.lazy import LazyStrategy
+        _strategies.add(strategy_definition.__name__)
 
-    @proxies(strategy_definition)
-    def accept(*args, **kwargs):
-        return LazyStrategy(strategy_definition, args, kwargs)
-    return accept
+        @proxies(strategy_definition)
+        def accept(*args, **kwargs):
+            result = LazyStrategy(strategy_definition, args, kwargs)
+            if force_reusable:
+                result.force_has_reusable_values = True
+                assert result.has_reusable_values
+            return result
+        return accept
+    return decorator
+
+
+defines_strategy = base_defines_strategy(False)
+defines_strategy_with_reusable_values = base_defines_strategy(True)
 
 
 class Nothing(SearchStrategy):
@@ -129,6 +138,9 @@ class Nothing(SearchStrategy):
     def do_draw(self, data):
         data.mark_invalid()
 
+    def calc_has_reusable_values(self):
+        return True
+
     def __repr__(self):
         return 'nothing()'
 
@@ -160,10 +172,7 @@ def just(value):
     """
     from hypothesis.searchstrategy.misc import JustStrategy
 
-    def calc_repr():
-        return 'just(%s)' % (repr(value),)
-
-    return ReprWrapperStrategy(JustStrategy(value), calc_repr)
+    return JustStrategy(value)
 
 
 @defines_strategy
@@ -191,7 +200,7 @@ def one_of(*args):
 
 
 @cacheable
-@defines_strategy
+@defines_strategy_with_reusable_values
 def integers(min_value=None, max_value=None):
     """Returns a strategy which generates integers (in Python 2 these may be
     ints or longs).
@@ -250,7 +259,7 @@ def booleans():
 
 
 @cacheable
-@defines_strategy
+@defines_strategy_with_reusable_values
 def floats(
     min_value=None, max_value=None, allow_nan=None, allow_infinity=None
 ):
@@ -359,7 +368,7 @@ def floats(
 
 
 @cacheable
-@defines_strategy
+@defines_strategy_with_reusable_values
 def complex_numbers():
     """Returns a strategy that generates complex numbers."""
     from hypothesis.searchstrategy.numbers import ComplexStrategy
@@ -627,7 +636,7 @@ def streaming(elements):
 
 
 @cacheable
-@defines_strategy
+@defines_strategy_with_reusable_values
 def characters(whitelist_categories=None, blacklist_categories=None,
                blacklist_characters=None, min_codepoint=None,
                max_codepoint=None, whitelist_characters=None):
@@ -702,7 +711,7 @@ def characters(whitelist_categories=None, blacklist_categories=None,
 
 
 @cacheable
-@defines_strategy
+@defines_strategy_with_reusable_values
 def text(
     alphabet=None,
     min_size=None, average_size=None, max_size=None
@@ -769,7 +778,7 @@ def from_regex(regex):
 
 
 @cacheable
-@defines_strategy
+@defines_strategy_with_reusable_values
 def binary(
     min_size=None, average_size=None, max_size=None
 ):
@@ -998,7 +1007,7 @@ def from_type(thing):
 
 
 @cacheable
-@defines_strategy
+@defines_strategy_with_reusable_values
 def fractions(min_value=None, max_value=None, max_denominator=None):
     """Returns a strategy which generates Fractions.
 
@@ -1094,7 +1103,7 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
 
 
 @cacheable
-@defines_strategy
+@defines_strategy_with_reusable_values
 def decimals(min_value=None, max_value=None,
              allow_nan=None, allow_infinity=None, places=None):
     """Generates instances of decimals.Decimal, which may be:
@@ -1221,7 +1230,7 @@ def permutations(values):
     return PermutationStrategy()
 
 
-@defines_strategy
+@defines_strategy_with_reusable_values
 @renamed_arguments(
     min_datetime='min_value',
     max_datetime='max_value',
@@ -1285,7 +1294,7 @@ def datetimes(
     return DatetimeStrategy(min_value, max_value, timezones)
 
 
-@defines_strategy
+@defines_strategy_with_reusable_values
 @renamed_arguments(
     min_date='min_value',
     max_date='max_value',
@@ -1305,7 +1314,7 @@ def dates(
     return DateStrategy(min_value, max_value)
 
 
-@defines_strategy
+@defines_strategy_with_reusable_values
 @renamed_arguments(
     min_time='min_value',
     max_time='max_value',
@@ -1332,7 +1341,7 @@ def times(
                      timezones=timezones).map(lambda t: t.timetz())
 
 
-@defines_strategy
+@defines_strategy_with_reusable_values
 @renamed_arguments(
     min_delta='min_value',
     max_delta='max_value',
@@ -1472,6 +1481,7 @@ def choices():
 
 
 @cacheable
+@defines_strategy_with_reusable_values
 def uuids():
     """Returns a strategy that generates UUIDs.
 
@@ -1480,13 +1490,12 @@ def uuids():
 
     """
     from uuid import UUID
-    return ReprWrapperStrategy(
-        shared(randoms(), key='hypothesis.strategies.uuids.generator').map(
-            lambda r: UUID(int=r.getrandbits(128))
-        ), 'uuids()')
+    return shared(randoms(), key='hypothesis.strategies.uuids.generator').map(
+        lambda r: UUID(int=r.getrandbits(128))
+    )
 
 
-@defines_strategy
+@defines_strategy_with_reusable_values
 def runner(default=not_set):
     """A strategy for getting "the current test runner", whatever that may be.
     The exact meaning depends on the entry point, but it will usually be the
@@ -1623,8 +1632,8 @@ def check_type(typ, arg, name=''):
 
 
 @check_function
-def check_strategy(arg):
-    check_type(SearchStrategy, arg)
+def check_strategy(arg, name=''):
+    check_type(SearchStrategy, arg, name)
 
 
 @check_function
